@@ -1,5 +1,3 @@
-/* $Xorg: Paned.c,v 1.4 2001/02/09 02:03:45 xorgcvs Exp $ */
-
 /***********************************************************
 
 Copyright 1987, 1988, 1994, 1998  The Open Group
@@ -46,7 +44,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/lib/Xaw/Paned.c,v 1.12 2001/12/14 19:54:41 dawes Exp $ */
+/* $XFree86: xc/lib/Xaw/Paned.c,v 1.14 2007/04/09 15:37:12 tsi Exp $ */
 
 /*
  * Updated and significantly modified from the Athena VPaned Widget.
@@ -1511,6 +1509,7 @@ GetPaneStack(PanedWidget pw, Bool shrink, Pane *pane, int *start_size)
 {
     if (pw->paned.stack == NULL) {
 	*pane = NULL; 
+	*start_size = 0;
 	return;
     }
 
